function blurflow_demo
%BLURFLOW_DEMO runs optical flow on a 4-image test sequence

% Travis Portz
% University of Wisconsin-Madison
% 2012

% change '8' to number of available cores
setenv('OMP_NUM_THREADS','8');

im0 = im2double(imread('image0.png'));
im1 = im2double(imread('image1.png'));
im2 = im2double(imread('image2.png'));
im3 = im2double(imread('image3.png'));

% exposure times as fractions of frame time
taus = [0.839317 0.839317 0.093240 0.140527];

params.alpha = 0.01;
params.gamma = 1;
params.epsilon_data = 0.001;
params.epsilon_spatial = 0.001;
params.max_outer_iterations = 16;
params.max_inner_iterations = 1;
params.max_cg_iterations = 50;
params.cg_threshold = 1e-8;
params.max_images = 240;
params.color_weight = 0.5;

if exist('baseline10.dat','file')
    baseline10 = flow_read('baseline10.dat');
else
    baseline10 = baseline_flow(im1,im0,params);
    flow_write(baseline10,'baseline10.dat');
end

if exist('baseline12.dat','file')
    baseline12 = flow_read('baseline12.dat');
else
    baseline12 = baseline_flow(im1,im2,params);
    flow_write(baseline12,'baseline12.dat');
end

if exist('baseline23.dat','file')
    baseline23 = flow_read('baseline23.dat');
else
    baseline23 = baseline_flow(im2,im3,params);
    flow_write(baseline23,'baseline23.dat');
end

refined12 = blurred_flow(im1,im2,taus(2),taus(3),params,baseline10,baseline12,baseline23);
flow_write(refined12,'refined12.dat');

figure; flow_viz(baseline12,25);
figure; flow_viz(refined12,25);
