%BLURRED_FLOW Computes blur-aware optical flow between two images.
%   UV12 = BLURRED_FLOW(IM1,IM2,TAU1,TAU2,PARAMS,UV10,UV12,UV23) computes
%   optical flow UV from IM1 to IM2 using a motion blur aware algorithm.
%
% IM1: RGB image.
% IM2: RGB image, same dimensions as IM1.
% TAU1: Exposure time of IM1 as fraction of frame time.
% TAU2: Exposure time of IM2 as fraction of frame time.
% PARAMS: Parameter structure. Fields:
%   scale_factor - pyramid scaling factor between levels (default: 0.75)
%   alpha - smoothness factor (default: 0.01)
%   gamma - gradient constancy factor (default: 1)
%   epsilon_data - robust penalty function parameter for data term (default: 0.001)
%   epsilon_spatial - robust penalty function parameter for smoothness term (default: 0.001)
%   num_pyramid_levels - number of image pyramid levels (default: minimum level width = 100)
%   max_outer_iterations - number of outer fixed-point iterations (default: 16)
%   max_inner_iterations - number of inner fixed-point iterations (default: 1)
%   max_cg_iterations - number of conjugate gradient iterations (default: 50)
%   cg_threshold - conjugate gradient termination threshold (default: 1e-8)
%   color_weight - weight for color component of feature image (default: 0.5)
%   max_images - maximum number of images in blurred image grid (default: 240)
%   range - range of valid flow vectors between IM1 and IM2 used for generating
%           blurred image grid, format: [u_min u_max v_min v_max]
%           (default: 105% of extent of baseline flow UV12)
% UV10: Baseline flow field from IM1 to previous frame.
% UV12: (Input) Baseline flow field from IM1 to IM2.
%       (Output) Blur-aware flow field from IM1 to IM2.
% UV23: Baseline flow field from IM2 to next frame.

% Travis Portz
% University of Wisconsin-Madison
% 2012