function uv = flow_concat(uv1,uv2)
%FLOW_CONCAT concatenates two flow fields
%   UV = FLOW_CONCAT(UV1,UV2) concatenates UV1 with UV2 to produce UV.

% Travis Portz
% University of Wisconsin-Madison
% 2012

u1 = uv1(:,:,1);
v1 = uv1(:,:,2);
u2 = uv2(:,:,1);
v2 = uv2(:,:,2);

[h,w,~] = size(u1);

[X,Y] = meshgrid(1:w,1:h);
XU = X + u1;
YV = Y + v1;
XU(XU > w) = w;
XU(XU < 1) = 1;
YV(YV > h) = h;
YV(YV < 1) = 1;

uv = zeros(h,w,2);
uv(:,:,1) = u1 + interp2(u2,XU,YV);
uv(:,:,2) = v1 + interp2(v2,XU,YV);
