function warped = flow_warp(uv12,img2)
%FLOW_WARP warps an image using an optical flow field
%   WARPED = FLOW_WARP(UV12,IMG2) uses the flow field UV12 from IMG1 to
%   IMG2 to warp IMG2 back to the location of IMG1.
%
% UV12: Flow field from IMG1 to IMG2.
% IMG2: Image to be warped.
% WARPED: (Optional) Warped version of IMG2. If no output argument is
%         specified, the image will be displayed in the current figure.

% Travis Portz
% University of Wisconsin-Madison
% 2012

u = uv12(:,:,1);
v = uv12(:,:,2);
img1 = zeros([size(u) 3]);
[h,w] = size(u);
[X,Y] = meshgrid(1:w,1:h);
Xi = X + u;
Yi = Y + v;
img1(:,:,1) = interp2(img2(:,:,1),Xi,Yi,'linear',0);
img1(:,:,2) = interp2(img2(:,:,2),Xi,Yi,'linear',0);
img1(:,:,3) = interp2(img2(:,:,3),Xi,Yi,'linear',0);

if nargout == 0
    imshow(img1);
else
    warped = img1;
end
