/*
 * baseline_flow.cpp
 *
 *  Created on: Jun 26, 2012
 *      Author: Travis Portz
 */

#include "blurflow.hpp"
#include "utility.hpp"

// uv12 = blurred_flow(im1, im2, params)
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if (nlhs != 1)
	{
		log_message("Error: expecting 1 LHS argument.\n");
		return;
	}
	if (nrhs != 3)
	{
		log_message("Error: expecting 3 RHS arguments.\n");
		return;
	}

	log_message("loading images...");
	image3f im1, im2;
	load_image(prhs[0], im1);
	load_image(prhs[1], im2);
	if (im1.width() != im2.width() || im1.height() != im2.height())
	{
		log_message("Error: image dimensions do not match.\n");
		return;
	}
	log_message(" done.\n");

	flow_params params(prhs[2]);

	if (mxGetFieldNumber(prhs[2], "num_pyramid_levels") == -1)
	{
		// Number of pyramid levels not specified. Set so that smallest level is at least 30 pixels wide.
		params.num_pyramid_levels = static_cast<int>(std::floor(std::log(30.0 / im1.width()) / std::log(params.scale_factor)));
	}

	flow32f_field flow12;
	compute_flow_baseline(im1, im2, params, flow12);
	
	flow_to_matlab(flow12, plhs[0]);
}
