/*
 * blur.cpp
 *
 *  Created on: Oct 19, 2011
 *      Author: Travis Portz
 */

#include "blur.hpp"
#include "poly.hpp"
#include "filter.hpp"
#include <sstream>

float compute_alpha(int n, double u_1, double u_n)
{
	std::vector<double> p(n, 0.0);
	p[0] = 1.0;
	p[n - 2] = u_1 - u_n;
	p[n - 1] = u_n - u_1 - 1.0;
	return static_cast<float>(largest_real_root(p));
}

void compute_axis_points(float u_min, float u_max, int n, std::vector<float>& points)
{
	if (u_min <= 0.0f && u_max >= 0.0f)
	{
		// The y-axis bisects the grid. We don't want the y-axis to bisect any cells, so we split it at 0.
		int n_right = (u_max > 1.0f) ? fast_round((n - 1) / (1.0f - u_min / u_max)) : 1;
		int n_left = n - 1 - n_right;
		if (static_cast<float>(n_left) >= -u_min)
		{
			// The spacing is less than or equal to 1. Force it to be as close to 1 as possible with uniform spacing.
			n_left = static_cast<int>(std::ceil(-u_min));
			float du = (-u_min - 1.0f) / (n_left - 1);
			float u = u_min;
			for (int i = 0; i < n_left; i++)
			{
				points.push_back(u);
				u += du;
			}
		}
		else if (n_left > 1)
		{
			// The spacing is more than 1. Make it smaller closer to 0.
			float a_left = compute_alpha(n_left, 1.0, -u_min);
			for (int i = n_left - 1; i >= 0; i--)
				points.push_back(-1.0f - (std::pow(a_left, i) - 1.0f) / (a_left - 1.0f));
		}
		else
		{
			points.push_back(u_min);
		}
		points.push_back(0.0f);
		if (static_cast<float>(n_right) >= u_max)
		{
			n_right = static_cast<int>(std::ceil(u_max));
			float du = (u_max - 1.0f) / (n_right - 1);
			float u = (n_right > 1) ? 1.0f : u_max;
			for (int i = 0; i < n_right; i++)
			{
				points.push_back(u);
				u += du;
			}
		}
		else if (n_right > 1)
		{
			float a_right = compute_alpha(n_right, 1.0, u_max);
			for (int i = 0; i < n_right; i++)
				points.push_back(1.0f + (std::pow(a_right, i) - 1.0f) / (a_right - 1.0f));
		}
		else
		{
			points.push_back(u_max);
		}
	}
	else if (u_min > 0.0f)
	{
		if (static_cast<float>(n - 1) >= u_max - u_min)
		{
			n = static_cast<int>(std::ceil(u_max - u_min)) + 1;
			float du = (u_max - u_min) / (n - 1);
			float u = u_min;
			for (int i = 0; i < n; i++)
			{
				points.push_back(u);
				u += du;
			}
		}
		else if (n > 1)
		{
			float a = compute_alpha(n, u_min, u_max);
			for (int i = 0; i < n; i++)
				points.push_back(u_min + (std::pow(a, i) - 1.0f) / (a - 1.0f));
		}
		else
		{
			points.push_back((u_min + u_max) * 0.5f);
		}
	}
	else // u_max < 0.0f
	{
		if (static_cast<float>(n - 1) >= u_max - u_min)
		{
			n = static_cast<int>(std::ceil(u_max - u_min)) + 1;
			float du = (u_max - u_min) / (n - 1);
			float u = u_min;
			for (int i = 0; i < n; i++)
			{
				points.push_back(u);
				u += du;
			}
		}
		else if (n > 1)
		{
			float a = compute_alpha(n, -u_max, -u_min);
			for (int i = n - 1; i >= 0; i--)
				points.push_back(u_max - (std::pow(a, i) - 1.0f) / (a - 1.0f));
		}
		else
		{
			points.push_back((u_min + u_max) * 0.5f);
		}
	}
}
