/*
 * blur.hpp
 *
 *  Created on: Oct 19, 2011
 *      Author: Travis Portz
 */

#ifndef BLUR_HPP
#define BLUR_HPP

#include "image.hpp"
#include "flow.hpp"
#include <vector>
#include <algorithm>
#include <limits>

const int D = 5;
typedef pixel<float, D> feature_pixel;
typedef image<feature_pixel> feature_image;

struct blurred_image_cell
{
	float u1, u2, v1, v2;
	int i, j, offset;
	float coeff;

	blurred_image_cell() : u1(std::numeric_limits<float>::infinity()), u2(std::numeric_limits<float>::infinity()),
			v1(std::numeric_limits<float>::infinity()), v2(std::numeric_limits<float>::infinity()), i(-1), j(-1), offset(-1) { }

	inline bool is_outside(float u, float v)
	{
		return (u < u1 || u > u2 || v < v1 || v > v2);
	}
};

// blur_positive_v: if v < 0, u = -u and v = |v|
// blur_positive_u: if u < 0, u = |u| and v = -v
// blur_full_range: no change
enum blur_range_mode { blur_positive_v, blur_positive_u, blur_full_range };

struct blur_range
{
	float u_min, u_max, v_min, v_max;
	blur_range_mode mode;

	blur_range negate()
	{
		blur_range range = { -u_max, -u_min, -v_max, -v_min, blur_full_range };
		return range;
	}

	static blur_range from_field(const flow32f_field& field)
	{
		blur_range range;
		range.u_min = std::numeric_limits<float>::infinity(); range.u_max = -range.u_min;
		range.v_min = range.u_min; range.v_max = -range.v_min;
		const flow32f* data = field.data();
		int n = field.width() * field.height();
		for (int i = 0; i < n; i++)
		{
			flow32f flow = data[i];
			if (flow.x > range.u_max)
				range.u_max = flow.x;
			if (flow.x < range.u_min)
				range.u_min = flow.x;
			if (flow.y > range.v_max)
				range.v_max = flow.y;
			if (flow.y < range.v_min)
				range.v_min = flow.y;
		}
		range.mode = blur_full_range;
		return range;
	}
};

void compute_axis_points(float u_min, float u_max, int n, std::vector<float>& points);

#endif /* BLUR_HPP */
