/*
 * blurflow.hpp
 *
 *  Created on: Oct 21, 2011
 *      Author: Travis Portz
 */

#ifndef BLURFLOW_HPP
#define BLURFLOW_HPP

#include "mex.h"
#include "image.hpp"
#include "flow.hpp"
#include "blurgrid.hpp"

struct flow_params
{
	float scale_factor; // pyramid scaling factor between levels
	float alpha; // smoothness factor
	float gamma; // gradient constancy factor
	float epsilon_data, epsilon_spatial; // robust cost function parameters
	int num_pyramid_levels;
	int max_outer_iterations, max_inner_iterations;
	int max_cg_iterations;
	float cg_threshold; // conjugate gradient convergence threshold (sigma^2)
	blur_range range;
	int max_images; // maximum number of images in blurred grid
	float color_weight; // weight for color component of feature image

	flow_params() { }
	flow_params(const mxArray* params);
};

// Compute the flow without blurring.
void compute_flow_baseline(image3f& from, image3f& to, flow_params params, flow32f_field& field);

// Compute the flow with blurring based on the current flow estimate and the pre-computed flows going out from the two images.
// flow10, flow12, and flow23 are baseline flows. The output field is a new estimate of flow12.
void compute_flow_blurred(image3f& from, image3f& to, float tau_from, float tau_to, flow_params params,
		const flow32f_field& flow10, const flow32f_field& flow12, const flow32f_field& flow23, flow32f_field& field);

#endif /* BLURFLOW_HPP */
