/*
 * blurred_flow.cpp
 *
 *  Created on: Jun 26, 2012
 *      Author: Travis Portz
 */

#include "blurflow.hpp"
#include "utility.hpp"

// uv12 = blurred_flow(im1, im2, tau1, tau2, params, uv10, uv12, uv23)
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if (nlhs != 1)
	{
		log_message("Error: expecting 1 LHS argument.\n");
		return;
	}
	if (nrhs != 8)
	{
		log_message("Error: expecting 8 RHS arguments.\n");
		return;
	}

	log_message("loading images...");
	image3f im1, im2;
	load_image(prhs[0], im1);
	load_image(prhs[1], im2);
	if (im1.width() != im2.width() || im1.height() != im2.height())
	{
		log_message("Error: image dimensions do not match.\n");
		return;
	}
	log_message(" done.\n");

	float tau1 = static_cast<float>(*static_cast<double*>(mxGetData(prhs[2])));
	float tau2 = static_cast<float>(*static_cast<double*>(mxGetData(prhs[3])));

	flow_params params(prhs[4]);

	log_message("loading flows...");
	flow32f_field flow10, flow12, flow23;
	flow_from_matlab(prhs[5], flow10);
	flow_from_matlab(prhs[6], flow12);
	flow_from_matlab(prhs[7], flow23);
	log_message(" done.\n");

	if (mxGetFieldNumber(prhs[4], "range") == -1)
	{
		// Blur range not specified, use extent of baseline flow to initialize.
		params.range = blur_range::from_field(flow12);
		// Expand range by 5%.
		float padding = 0.025f * (params.range.u_max - params.range.u_min);
		params.range.u_min -= padding;
		params.range.u_max += padding;
		padding = 0.025f * (params.range.v_max - params.range.v_min);
		params.range.v_min -= padding;
		params.range.v_max += padding;
	}

	if (mxGetFieldNumber(prhs[4], "num_pyramid_levels") == -1)
	{
		// Number of pyramid levels not specified. Set so that smallest level is at least 100 pixels wide.
		params.num_pyramid_levels = static_cast<int>(std::floor(std::log(100.0 / im1.width()) / std::log(params.scale_factor)));
	}

	flow32f_field flow12_refined;
	compute_flow_blurred(im1, im2, tau1, tau2, params, flow10, flow12, flow23, flow12_refined);

	flow_to_matlab(flow12_refined, plhs[0]);
}
