/*
 * image.cpp
 *
 *  Created on: Oct 18, 2011
 *      Author: Travis Portz
 */

#include "image.hpp"

void load_image(const mxArray* src, image3f& dst)
{
	const mwSize* dims = mxGetDimensions(src);
	assert(dims[2] == 3); // Input should be de-interleaved RGB.
	dst.allocate(dims[1], dims[0]); // Converting from column-major to row-major.

	pixel3f* dst_ptr = dst.data();

	if (mxGetClassID(src) == mxDOUBLE_CLASS)
	{
		const double* src_ptr = static_cast<const double*>(mxGetData(src));
		for (int c = 0; c < 3; c++)
			for (int x = 0; x < dims[1]; x++)
				for (int y = 0; y < dims[0]; y++)
					dst(x, y)[c] = static_cast<float>(*(src_ptr++));
	}
	else if (mxGetClassID(src) == mxUINT8_CLASS)
	{
		const unsigned char* src_ptr = static_cast<const unsigned char*>(mxGetData(src));
		for (int c = 0; c < 3; c++)
			for (int x = 0; x < dims[1]; x++)
				for (int y = 0; y < dims[0]; y++)
					dst(x, y)[c] = static_cast<float>(*(src_ptr++)) / 255.0f;
	}
}

void save_image(mxArray* dst, const image3f& src)
{
	mwSize dims[3] = { src.height(), src.width(), 3 };
	dst = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL);

	double* dst_ptr = static_cast<double*>(mxGetData(dst));
	for (int c = 0; c < 3; c++)
		for (int x = 0; x < dims[1]; x++)
			for (int y = 0; y < dims[0]; y++)
				*(dst_ptr++) = src(x, y)[c];
}

template <>
void desaturate<float>(const image3f& src, image1f& dst)
{
	dst.allocate(src);
	int n = src.width() * src.height();
	const pixel3f* src_data = src.data();
	float* dst_data = dst.data();
	for (int i = 0; i < n; i++)
	{
		pixel3f p = src_data[i];
		dst_data[i] = 0.2989f * p[0] + 0.5870f * p[1] + 0.1140f * p[2];
	}
}
