/*
 * pixel.hpp
 *
 *  Created on: Oct 17, 2011
 *      Author: Travis Portz
 */

#ifndef PIXEL_HPP
#define PIXEL_HPP

template <typename T, unsigned C>
class pixel
{
public:
	pixel() { }
	pixel(T v)
	{
		for (unsigned i = 0; i < C; i++)
			data[i] = v;
	}
	pixel(const pixel<T,C>& other)
	{
		for (unsigned i = 0; i < C; i++)
			data[i] = other.data[i];
	}

	pixel<T,C>& operator=(const pixel<T,C>& other)
	{
		for (unsigned i = 0; i < C; i++)
			data[i] = other.data[i];
		return *this;
	}

	T operator[](unsigned c) const { return data[c]; }
	T& operator[](unsigned c) { return data[c]; }

	pixel<T,C>& operator+=(const pixel<T,C>& other)
	{
		for (unsigned i = 0; i < C; i++)
			data[i] += other.data[i];
		return *this;
	}
	const pixel<T,C> operator+(const pixel<T,C>& other) const
	{
		return pixel<T,C>(*this) += other; // RVO
	}

	pixel<T,C>& operator*=(const pixel<T,C>& other)
	{
		for (unsigned i = 0; i < C; i++)
			data[i] *= other.data[i];
		return *this;
	}
	const pixel<T,C> operator*(const pixel<T,C>& other) const
	{
		return pixel<T,C>(*this) *= other; // RVO
	}

	template <typename Scalar> pixel<T,C>& operator*=(Scalar s)
	{
		for (unsigned i = 0; i < C; i++)
			data[i] *= s;
		return *this;
	}
	template <typename Scalar> const pixel<T,C> operator*(Scalar s) const
	{
		return pixel<T,C>(*this) *= s; // RVO
	}

	pixel<T,C>& operator-=(const pixel<T,C>& other)
	{
		for (unsigned i = 0; i < C; i++)
			data[i] -= other.data[i];
		return *this;
	}
	const pixel<T,C> operator-(const pixel<T,C>& other) const
	{
		return pixel<T,C>(*this) -= other; // RVO
	}
	const pixel<T,C> operator-() const
	{
		pixel<T,C> result;
		for (unsigned i = 0; i < C; i++)
			result.data[i] = -data[i];
		return result;
	}

	template <typename Scalar>
	Scalar length_squared() const
	{
		Scalar result = 0;
		for (unsigned i = 0; i < C; i++)
			result += static_cast<Scalar>(data[i] * data[i]);
		return result;
	}

	// Converts to scalar by summing channels
	T collapse() const
	{
		double sum = 0;
		for (unsigned i = 0; i < C; i++)
			sum += static_cast<double>(data[i]);
		return static_cast<T>(sum / C);
	}

private:
	T data[C];
};

typedef pixel<float, 3> pixel3f;
typedef pixel<unsigned char, 3> pixel3c;

#endif /* PIXEL_HPP */
