/*
 * utility.hpp
 *
 *  Created on: Aug 11, 2011
 *      Author: Travis Portz
 */

#ifndef UTILITY_HPP
#define UTILITY_HPP

#include "mex.h"

#ifndef nullptr
#define nullptr 0
#endif

inline int fast_round(double x)
{
   return static_cast<int>(x > 0.0 ? x + 0.5 : x - 0.5);
}

inline int fast_round(float x)
{
   return static_cast<int>(x > 0.0f ? x + 0.5f : x - 0.5f);
}

template <typename T>
inline T clamp(T value, T min_value, T max_value)
{
	return std::min(std::max(value, min_value), max_value);
}

// 2d coordinate
struct coord2
{
	int x, y;

	coord2(int x, int y) : x(x), y(y) { }
	coord2(const coord2& other) : x(other.x), y(other.y) { }

	inline coord2& operator=(const coord2& other)
	{
		x = other.x;
		y = other.y;
		return *this;
	}

	inline bool in_bounds(int width, int height) const
	{
		return (x >= 0 && x < width && y >= 0 && y < height);
	}

	// Max values are exclusive
	inline bool out_of_bounds(int x_min, int x_max, int y_min, int y_max)
	{
		return (x < x_min || x >= x_max || y < y_min || y >= y_max);
	}

	inline bool operator==(const coord2& other) const
	{
		return (x == other.x && y == other.y);
	}
};

// 2d floating-point coordinate
struct coord2f
{
	float x, y;

	coord2f(float x, float y) : x(x), y(y) { }
	coord2f(const coord2f& other) : x(other.x), y(other.y) { }

	inline coord2f& operator=(const coord2f& other)
	{
		x = other.x;
		y = other.y;
		return *this;
	}

	inline bool in_bounds(float width, float height) const
	{
		return (x >= 0.0f && x <= (width - 1.0f) && y >= 0.0f && y <= (height - 1.0f));
	}

	inline bool operator==(const coord2f& other) const
	{
		return (x == other.x && y == other.y);
	}
};

#include <stdarg.h>

inline void log_message(const char* fmt, ...)
{
	va_list args;
	va_start(args, fmt);
	char str[256];
	vsprintf(str, fmt, args);
	mexPrintf(str);
	mexEvalString("drawnow;");
}

#endif /* UTILITY_HPP */
